let int_of_hex c = int_of_string (String.concat "" ["0x"; String.make 1 c])

let solve h w input = 
  let (hh,ww) = (h+2,4*w+2) in
  let picture = Array.make_matrix hh ww false in
  let _ =
    for i=1 to hh-2 do
      for j=1 to ww-2 do
	picture.(i).(j) <- (((int_of_hex input.(i-1).[(j-1)/4]) lsr (3-((j-1) mod 4))) land 1) = 1;
      done
    done
  in
      (* We just constructed the boolean picture array.  We use this from now on. *)

  let mark = Array.make_matrix hh ww (-1) in

  let rec dfs y x t c = 
    if picture.(y).(x) = t && mark.(y).(x) < 0 then begin
      mark.(y).(x) <- c;
      if x>0 then dfs y (x-1) t c;
      if x<ww-1 then dfs y (x+1) t c;
      if y>0 then dfs (y-1) x t c;
      if y<hh-1 then dfs (y+1) x t c;
    end
  in

  let next (y,x) = ((if x<ww-1 then y else y+1), (x+1) mod ww) in

  let rec scan (y,x) c =
    if y=hh then c else begin
      if mark.(y).(x) >= 0 then scan (next (y,x)) c else begin
	dfs y x picture.(y).(x) c;
	scan (next (y,x)) (c+1)
      end
    end
  in 
  let c = scan (0,0) 0 in

(*  There are c colors 0...c-1.  Imagine these colors as vertices,
    and build an adjacency matrix.  Then we can determine for each 
    black region how many different white regions it touches, and
    this will tell us what kind of character it is *)

  let adj = Array.make_matrix c c false in
  let blackset = Array.make c false in

  let _ = 
    for y=0 to hh-1 do for x=0 to ww-1 do
      if y+1<hh then adj.(mark.(y).(x)).(mark.(y+1).(x)) <- true;
      if x+1<ww then adj.(mark.(y).(x)).(mark.(y).(x+1)) <- true;
      if picture.(y).(x) then blackset.(mark.(y).(x)) <- true
    done done
  in
      
  let rec count_neighbors me col accum = 
    if col=c then accum else
      count_neighbors me (col+1) 
	(accum + (if me !=col && (adj.(me).(col) || adj.(col).(me)) then 1 else 0))
  in

  let rec build_answer col accum = 
    if col=c then accum
    else if not blackset.(col) then build_answer (col+1) accum
    else 
	let sym = match count_neighbors col 0 0 with 
	  | 1 -> "W"
	  | 2 -> "A"
	  | 3 -> "K"
	  | 4 -> "J"
	  | 5 -> "S"
	  | 6 -> "D"
	  | _ -> "?"
	in
	  build_answer (col+1) (sym::accum)
  in
    String.concat "" (List.sort compare (build_answer 0 []))
    
let read_int () = 
  Scanf.bscanf Scanf.Scanning.stdib " %d " (fun x -> x)

let read_string () = 
  Scanf.bscanf Scanf.Scanning.stdib " %s " (fun x -> x)

let rec solve_rest case = 
  let h = read_int() in
  let w = read_int() in
    if h>0 then
      let input = Array.make h "" in
	for i=0 to h-1 do
	  input.(i) <- read_string()
	done;
	Printf.printf "Case %d: %s\n" case (solve h w input);
	solve_rest (case+1)
;;

solve_rest 1
